@@ -37,6 +37,8 @@ class Variable:
         self.read_only_indicator = None
         self.has_del = False
 
+        self.version_number = 0
+
     def getName( self ):
         return self.variable_name
 
@@ -75,6 +77,10 @@ class Variable:
     def setHasDelIndicator( self ):
         self.has_del = True
 
+    def allocateTargetNumber( self ):
+        self.version_number += 1
+
+        return self.version_number
 
     # pylint: disable=R0201
     def isLocalVariable( self ):
@@ -150,9 +156,6 @@ class Variable:
         # Need to provider a reference class, or else making references cannot work.
         assert self.reference_class, self
 
-        if self.__class__ == TempVariableReference:
-            assert self.reference_class == TempVariableReference2, ( self.__class__, self.reference_class )
-
         # Search for existing references to be re-used before making a new one.
         for reference in self.references:
             if reference.getOwner() is owner:
@@ -176,7 +179,7 @@ class Variable:
 
     def getCodeName( self ):
         # Abstract method, pylint: disable=R0201
-        assert False
+        assert False, self
 
 
 class VariableReferenceBase( Variable ):
@@ -199,7 +202,10 @@ class VariableReferenceBase( Variable ):
         return self.getReferenced().read_only_indicator
 
     def __repr__( self ):
-        return "<%s to %s>" % ( self.__class__.__name__, str( self.variable )[1:-1] )
+        return "<%s to %s>" % (
+            self.__class__.__name__,
+            str( self.variable )[1:-1]
+        )
 
     def getReferenced( self ):
         return self.variable
@@ -278,6 +284,15 @@ class ModuleVariableReference( VariableReferenceBase ):
         )
 
         self.global_statement = False
+        self.exec_statement = False
+
+    def __repr__( self ):
+        return "<ModuleVariableReference '%s' of '%s'%s%s>" % (
+            self.variable_name,
+            self.getReferenced().getModuleName(),
+            " from global statement" if self.global_statement else "",
+            " from exec statement" if self.exec_statement else "",
+        )
 
     def markFromGlobalStatement( self ):
         self.global_statement = True
@@ -285,6 +300,12 @@ class ModuleVariableReference( VariableReferenceBase ):
     def isFromGlobalStatement( self ):
         return self.global_statement
 
+    def markFromExecStatement( self ):
+        self.exec_statement = True
+
+    def isFromExecStatement( self ):
+        return self.exec_statement
+
     def isModuleVariableReference( self ):
         return True
 
@@ -530,6 +551,14 @@ class TempVariable( Variable ):
 
         self.needs_free = needs_free
 
+    def isDeclared( self ):
+        return self.declared
+
+    def markAsDeclared( self ):
+        assert not self.declared
+
+        self.declared = True
+
     def getDeclarationTypeCode( self, in_context ):
         assert self.needs_free is not None, self
 
