@@ -20,21 +20,21 @@
 """
 
 from .Identifiers import (
-    Identifier,
+    SpecialConstantIdentifier,
     ConstantIdentifier,
-    LocalVariableIdentifier,
-    ClosureVariableIdentifier,
-    SpecialConstantIdentifier
+    Identifier
 )
 
 from .Namify import namifyConstant
 
-from nuitka.Constants import HashableConstant
+from nuitka.Constants import HashableConstant, constant_builtin_types
 
 from nuitka.__past__ import iterItems
 
 from nuitka.Utils import python_version
 
+from nuitka import Options
+
 # False alarms about "hashlib.md5" due to its strange way of defining what is
 # exported, pylint won't understand it. pylint: disable=E1101
 
@@ -63,14 +63,19 @@ class PythonContextBase:
 
         self.try_finally_counts = []
 
+        self.call_temp_count = 0
+
+    def isPythonModule( self ):
+        return False
+
+    def hasLocalsDict( self ):
+        return False
+
     def allocateTryNumber( self ):
         self.try_count += 1
 
         return self.try_count
 
-    def hasLocalsDict( self ):
-        return False
-
     def setTryFinallyCount( self, value ):
         self.try_finally_counts.append( value )
 
@@ -83,6 +88,11 @@ class PythonContextBase:
         else:
             return None
 
+    def allocateCallTempNumber( self ):
+        self.call_temp_count += 1
+
+        return self.call_temp_count
+
 
 class PythonChildContextBase( PythonContextBase ):
     def __init__( self, parent ):
@@ -90,33 +100,12 @@ class PythonChildContextBase( PythonContextBase ):
 
         self.parent = parent
 
-    def addEvalOrderUse( self, value ):
-        self.parent.addEvalOrderUse( value )
-
-    def addMakeTupleUse( self, value ):
-        self.parent.addMakeTupleUse( value )
-
-    def addMakeListUse( self, value ):
-        self.parent.addMakeListUse( value )
-
-    def addMakeDictUse( self, value ):
-        self.parent.addMakeDictUse( value )
-
-    def getParent( self ):
-        return self.parent
-
     def getConstantHandle( self, constant ):
         return self.parent.getConstantHandle( constant )
 
-    def addFunctionCodes( self, code_name, function_decl, function_code ):
-        self.parent.addFunctionCodes( code_name, function_decl, function_code )
-
     def addGlobalVariableNameUsage( self, var_name ):
         self.parent.addGlobalVariableNameUsage( var_name )
 
-    def addExportDeclarations( self, declarations ):
-        self.parent.addExportDeclarations( declarations )
-
     def getModuleCodeName( self ):
         return self.parent.getModuleCodeName()
 
@@ -135,86 +124,119 @@ class PythonChildContextBase( PythonContextBase ):
             is_optimized  = is_optimized
         )
 
+    def addHelperCode( self, key, code ):
+        self.parent.addHelperCode( key, code )
+
+    def addDeclaration( self, key, code ):
+        self.parent.addDeclaration( key, code )
+
+
+def _getConstantDefaultPopulation():
+    result = (
+        # Basic values that the helper code uses all the times.
+        (),
+        {},
+        "",
+        True,
+        False,
+        0,
+
+        # For Python3 empty bytes, no effect for Python2, same as "", used for
+        # code objects.
+        b"",
+
+        # Python mechanics, used in various helpers.
+        "__module__",
+        "__class__",
+        "__name__",
+        "__metaclass__",
+        "__dict__",
+        "__doc__",
+        "__file__",
+        "__enter__",
+        "__exit__",
+        "__builtins__",
+        "__all__",
+
+        # Patched module name.
+        "inspect",
+
+        # Names of builtins used in helper code.
+        "compile",
+        "range",
+        "open",
+        "__import__",
+
+        # COMPILE_CODE uses read/strip method lookups.
+        "read",
+        "strip",
+
+    )
+
+    # For Python3 modules
+    if python_version >= 300:
+        result += ( "__cached__",  )
+
+    # For Python3 print
+    if python_version >= 300:
+        result += ( "print", "end", "file" )
+
+    if python_version >= 330:
+        result += ( "__loader__", )
+
+    # For patching Python2 internal class type
+    if python_version < 300:
+        result += (
+            "__getattr__",
+            "__setattr__",
+            "__delattr__"
+        )
+
+    # For patching Python2 sys attributes for current exception
+    if python_version < 300:
+        result += (
+            "exc_type",
+            "exc_value",
+            "exc_traceback"
+        )
+
+    # Executables only
+    if not Options.shallMakeModule():
+        result += ( "__main__", )
+
+    # Builtin original values
+    if not Options.shallMakeModule():
+        result += (
+            "type",
+            "len",
+            "range",
+            "repr",
+            "int",
+            "iter",
+        )
+
+        if python_version < 300:
+            result += (
+                "long",
+            )
+
+    return result
+
 
 class PythonGlobalContext:
     def __init__( self ):
         self.constants = {}
 
-        # Basic values that the code uses all the times.
-        self.getConstantHandle( () )
-        self.getConstantHandle( {} )
-        self.getConstantHandle( "" )
-        self.getConstantHandle( True )
-        self.getConstantHandle( False )
-        self.getConstantHandle( 0 )
-        # For Python3 empty bytes, no effect for Python2, same as ""
-        self.getConstantHandle( b"" )
-
-        # Python mechanics.
-        self.getConstantHandle( "__module__" )
-        self.getConstantHandle( "__class__" )
-        self.getConstantHandle( "__name__" )
-        self.getConstantHandle( "__metaclass__" )
-        self.getConstantHandle( "__dict__" )
-        self.getConstantHandle( "__doc__" )
-        self.getConstantHandle( "__file__" )
-        self.getConstantHandle( "__enter__" )
-        self.getConstantHandle( "__exit__" )
-        self.getConstantHandle( "__builtins__" )
-        self.getConstantHandle( "__all__" )
-
-        # For Python3 modules
-        if python_version >= 300:
-            self.getConstantHandle( "__cached__" )
-
-        if python_version >= 330:
-            self.getConstantHandle( "__loader__" )
-
-        # For patching Python2 internal class type
-        if python_version < 300:
-            self.getConstantHandle( "__getattr__" )
-            self.getConstantHandle( "__setattr__" )
-            self.getConstantHandle( "__delattr__" )
-
-        # Patched module name.
-        self.getConstantHandle( "inspect" )
-
-        # Names of builtins used in helper code.
-        self.getConstantHandle( "compile" )
-        self.getConstantHandle( "range" )
-        self.getConstantHandle( "open" )
-        self.getConstantHandle( "print" )
-        self.getConstantHandle( "__import__" )
-
-        # The print builtin needs some argument names.
-        self.getConstantHandle( "end" )
-        self.getConstantHandle( "file" )
-
-        # COMPILE_CODE uses read/strip method lookups.
-        self.getConstantHandle( "read" )
-        self.getConstantHandle( "strip" )
-
-        # Executables only
-        self.getConstantHandle( "__main__" )
-
-        # sys exception attributes
-        self.getConstantHandle( "exc_type" )
-        self.getConstantHandle( "exc_value" )
-        self.getConstantHandle( "exc_traceback" )
-
-        # Have EVAL_ORDER for 1..6 in any case, so we can use it in the C++ code freely
-        # without concern.
-        self.make_tuples_used = set( range( 1, 6 ) )
-        self.make_lists_used = set( range( 0, 1 ) )
-        self.make_dicts_used = set( range( 0, 3 ) )
-
-        # Include 0, so list and tuple making can use it without being special.
-        self.eval_orders_used = set( range( 0, 6 ) )
+        for value in _getConstantDefaultPopulation():
+            self.getConstantHandle( value )
 
         # Code objects needed.
         self.code_objects = {}
 
-    def getConstantHandle( self, constant ):
+    def getConstantHandle( self, constant, real_use = True ):
+        # There are many branches, each supposed to return.
+        # pylint: disable=R0911
+
         if constant is None:
             return SpecialConstantIdentifier( None )
         elif constant is True:
@@ -223,20 +245,49 @@ class PythonGlobalContext:
             return SpecialConstantIdentifier( False )
         elif constant is Ellipsis:
             return SpecialConstantIdentifier( Ellipsis )
+        elif constant in constant_builtin_types:
+            type_name = constant.__name__
+
+            if constant is int and python_version >= 300:
+                type_name = "long"
+
+            if constant is str and python_version < 300:
+                type_name = "string"
+
+            if constant is str and python_version > 300:
+                type_name = "unicode"
+
+            return Identifier(
+                "(PyObject *)&Py%s_Type" % type_name.title(),
+                0
+            )
         else:
-            key = ( type( constant ), HashableConstant( constant ) )
+            if real_use:
+                key = ( type( constant ), HashableConstant( constant ) )
 
-            if key not in self.constants:
-                self.constants[ key ] = "_python_" + namifyConstant( constant )
+                if key not in self.constants:
+                    self.constants[ key ] = "_python_" + namifyConstant(
+                        constant
+                    )
 
-            return ConstantIdentifier( self.constants[ key ], constant )
+                return ConstantIdentifier( self.constants[ key ], constant )
+            else:
+                return Identifier( "_python_" + namifyConstant( constant ), 0 )
 
     def getConstants( self ):
-        return sorted( self.constants.items(), key = lambda x: x[1] )
+        return self.constants
 
-    def getCodeObjectHandle( self, filename, code_name, line_number, arg_names, kw_only_count,
-                             is_generator, is_optimized ):
-        key = ( filename, code_name, line_number, arg_names, kw_only_count, is_generator, is_optimized )
+    def getCodeObjectHandle( self, filename, code_name, line_number, arg_names,
+                             kw_only_count, is_generator, is_optimized ):
+        key = (
+            filename,
+            code_name,
+            line_number,
+            arg_names,
+            kw_only_count,
+            is_generator,
+            is_optimized
+        )
 
         if key not in self.code_objects:
             self.code_objects[ key ] = Identifier(
@@ -253,41 +304,6 @@ class PythonGlobalContext:
     def getCodeObjects( self ):
         return sorted( iterItems( self.code_objects ) )
 
-    def addEvalOrderUse( self, value ):
-        assert type( value ) is int
-
-        self.eval_orders_used.add( value )
-
-    def getEvalOrdersUsed( self ):
-        return sorted( self.eval_orders_used )
-
-    def addMakeTupleUse( self, value ):
-        assert type( value ) is int
-
-        self.addEvalOrderUse( value ) # generated code uses it
-        self.make_tuples_used.add( value )
-
-    def addMakeListUse( self, value ):
-        assert type( value ) is int
-
-        self.addEvalOrderUse( value ) # generated code uses it
-        self.make_lists_used.add( value )
-
-    def addMakeDictUse( self, value ):
-        assert type( value ) is int
-
-        self.addEvalOrderUse( value * 2 ) # generated code uses it
-        self.make_dicts_used.add( value )
-
-    def getMakeTuplesUsed( self ):
-        return sorted( self.make_tuples_used )
-
-    def getMakeListsUsed( self ):
-        return sorted( self.make_lists_used )
-
-    def getMakeDictsUsed( self ):
-        return sorted( self.make_dicts_used )
-
 
 class PythonModuleContext( PythonContextBase ):
     # Plent of attributes, because it's storing so many different things.
@@ -302,31 +318,28 @@ class PythonModuleContext( PythonContextBase ):
 
         self.global_context = global_context
 
-        self.function_codes = {}
+        self.declaration_codes = {}
+        self.helper_codes = {}
 
         self.global_var_names = set()
 
-        self.temp_keepers = {}
-
-        self.export_declarations = []
-
     def __repr__( self ):
         return "<PythonModuleContext instance for module %s>" % self.filename
 
+    def isPythonModule( self ):
+        return True
+
     def getFrameHandle( self ):
         return Identifier( "frame_guard.getFrame()", 1 )
 
     def getFrameGuardClass( self ):
         return "FrameGuard"
 
-    def getParent( self ):
-        return None
-
     def getConstantHandle( self, constant ):
         return self.global_context.getConstantHandle( constant )
 
-    def getCodeObjectHandle( self, filename, code_name, line_number, arg_names, kw_only_count,
-                             is_generator, is_optimized ):
+    def getCodeObjectHandle( self, filename, code_name, line_number, arg_names,
+                             kw_only_count, is_generator, is_optimized ):
         return self.global_context.getCodeObjectHandle(
             filename      = filename,
             code_name     = code_name,
@@ -337,14 +350,6 @@ class PythonModuleContext( PythonContextBase ):
             is_optimized  = is_optimized
         )
 
-    def addFunctionCodes( self, code_name, function_decl, function_code ):
-        assert code_name not in self.function_codes, code_name
-
-        self.function_codes[ code_name ] = ( function_decl, function_code )
-
-    def getFunctionsCodes( self ):
-        return self.function_codes
-
     def getName( self ):
         return self.name
 
@@ -353,7 +358,7 @@ class PythonModuleContext( PythonContextBase ):
     def getModuleCodeName( self ):
         return self.code_name
 
-    # There cannot ne local variable in modules no need to consider the name.
+    # There cannot be local variable in modules no need to consider the name.
     # pylint: disable=W0613
     def hasLocalVariable( self, var_name ):
         return False
@@ -368,38 +373,29 @@ class PythonModuleContext( PythonContextBase ):
     def getGlobalVariableNames( self ):
         return sorted( self.global_var_names )
 
-    def addEvalOrderUse( self, value ):
-        self.global_context.addEvalOrderUse( value )
-
-    def addMakeTupleUse( self, value ):
-        self.global_context.addMakeTupleUse( value )
-
-    def addMakeListUse( self, value ):
-        self.global_context.addMakeListUse( value )
-
-    def addMakeDictUse( self, value ):
-        self.global_context.addMakeDictUse( value )
-
-    def addExportDeclarations( self, declarations ):
-        self.export_declarations.append( declarations )
-
-    def addTempKeeperUsage( self, variable_name, ref_count ):
-        self.temp_keepers[ variable_name ] = ref_count
-
-    def getTempKeeperRefCount( self, variable_name ):
-        return self.temp_keepers[ variable_name ]
-
-    def getTempKeeperUsages( self ):
-        return self.temp_keepers
-
-    def getExportDeclarations( self ):
-        return "\n".join( self.export_declarations )
-
     def setFrameGuardMode( self, guard_mode ):
         assert guard_mode == "once"
 
     def getReturnCode( self ):
-        return "return MOD_RETURN_VALUE( _module_%s );" % self.getModuleCodeName()
+        return "return MOD_RETURN_VALUE( _module_%s );" % (
+            self.getModuleCodeName()
+        )
+
+    def addHelperCode( self, key, code ):
+        assert key not in self.helper_codes
+
+        self.helper_codes[ key ] = code
+
+    def getHelperCodes( self ):
+        return self.helper_codes
+
+    def addDeclaration( self, key, code ):
+        assert key not in self.declaration_codes
+
+        self.declaration_codes[ key ] = code
+
+    def getDeclarations( self ):
+        return self.declaration_codes
 
 
 class PythonFunctionContext( PythonChildContextBase ):
@@ -408,14 +404,10 @@ class PythonFunctionContext( PythonChildContextBase ):
 
         self.function = function
 
-        self.needs_creation = function.needsCreation()
-
         # Make sure the local names are available as constants
         for local_name in function.getLocalVariableNames():
             self.getConstantHandle( constant = local_name )
 
-        self.temp_keepers = {}
-
         self.guard_mode = None
 
     def __repr__( self ):
@@ -424,6 +416,9 @@ class PythonFunctionContext( PythonChildContextBase ):
             self.function.getName()
         )
 
+    def getFunction( self ):
+        return self.function
+
     def hasLocalsDict( self ):
         return self.function.hasLocalsDict()
 
@@ -455,38 +450,11 @@ class PythonFunctionContext( PythonChildContextBase ):
         else:
             assert False, (self, self.guard_mode)
 
-    def getLocalHandle( self, var_name ):
-        return LocalVariableIdentifier( var_name, from_context = self.function.isGenerator() )
-
-    def getClosureHandle( self, var_name ):
-        if self.needs_creation:
-            if self.function.isGenerator():
-                return ClosureVariableIdentifier( var_name, from_context = "_python_context->common_context->" )
-            else:
-                return ClosureVariableIdentifier( var_name, from_context = "_python_context->" )
-        else:
-            if self.function.isGenerator():
-                return ClosureVariableIdentifier( var_name, from_context = "_python_context->" )
-            else:
-                return ClosureVariableIdentifier( var_name, from_context = "" )
-
-    def addTempKeeperUsage( self, variable_name, ref_count ):
-        self.temp_keepers[ variable_name ] = ref_count
-
-    def getTempKeeperRefCount( self, variable_name ):
-        return self.temp_keepers[ variable_name ]
-
-    def getTempKeeperUsages( self ):
-        return self.temp_keepers
-
 
 class PythonFunctionDirectContext( PythonFunctionContext ):
     def isForDirectCall( self ):
         return True
 
-    def getExportScope( self ):
-        return "NUITKA_CROSS_MODULE" if self.function.isCrossModuleUsed() else "NUITKA_LOCAL_MODULE"
-
     def isForCrossModuleUsage( self ):
         return self.function.isCrossModuleUsed()
 
@@ -500,3 +468,49 @@ class PythonFunctionCreatedContext( PythonFunctionContext ):
 
     def isForCreatedFunction( self ):
         return True
+
+
+class PythonStatementContext( PythonChildContextBase ):
+    def __init__( self, parent ):
+        PythonChildContextBase.__init__( self, parent = parent )
+
+        self.temp_keepers = {}
+
+    def getFrameHandle( self ):
+        return self.parent.getFrameHandle()
+
+    def getFrameGuardClass( self ):
+        return self.parent.getFrameGuardClass()
+
+    def hasLocalsDict( self ):
+        return self.parent.hasLocalsDict()
+
+    def isPythonModule( self ):
+        return self.parent.isPythonModule()
+
+    def getFunction( self ):
+        return self.parent.getFunction()
+
+    def allocateCallTempNumber( self ):
+        return self.parent.allocateCallTempNumber()
+
+    def addTempKeeperUsage( self, variable_name, ref_count ):
+        self.temp_keepers[ variable_name ] = ref_count
+
+    def getTempKeeperRefCount( self, variable_name ):
+        return self.temp_keepers[ variable_name ]
+
+    def getTempKeeperUsages( self ):
+        return self.temp_keepers
+
+    def allocateTryNumber( self ):
+        return self.parent.allocateTryNumber()
+
+    def setTryFinallyCount( self, value ):
+        self.parent.setTryFinallyCount( value )
+
+    def removeFinallyCount( self ):
+        self.parent.removeFinallyCount()
+
+    def getTryFinallyCount( self ):
+        return self.parent.getTryFinallyCount()
