@@ -25,6 +25,7 @@ and to manage them.
 
 # Python3 compatibility.
 from nuitka.__past__ import iterItems
+from nuitka.oset import OrderedSet
 
 from nuitka.nodes import ValueFriends
 
@@ -36,7 +37,66 @@ from nuitka.nodes.NodeMakingHelpers import (
 from nuitka import Options, Utils, Importing
 from nuitka.tree import Recursion
 
-from logging import debug
+from logging import debug, warning
+
+
+class VariableTrace:
+    def __init__( self, variable, version ):
+        self.variable     = variable
+        self.version      = version
+
+        self.usages = []
+
+    def addUsage( self, ref_node ):
+        self.usages.append( ref_node )
+
+
+class VariableAssignTrace( VariableTrace ):
+    def __init__( self, target_node, variable, version, value_friend ):
+        VariableTrace.__init__(
+            self,
+            variable = variable,
+            version  = version
+        )
+
+        self.target_node = target_node
+        self.value_friend = value_friend
+
+    def onValueEscape( self ):
+        # TODO: Tell value friend to degrade intelligently.
+        self.value_friend = None
+
+
+class VariableReferenceTrace( VariableTrace ):
+    def __init__( self, ref_node, variable, version ):
+        VariableTrace.__init__(
+            self,
+            variable = variable,
+            version  = version
+        )
+
+        self.ref_node = ref_node
+
+        self.usages.append( ref_node )
+
+    def onValueEscape( self ):
+        pass
+
+
+class VariableMergeTrace( VariableTrace ):
+    def __init__( self, variable, version, trace_yes, trace_no ):
+        VariableTrace.__init__(
+            self,
+            variable = variable,
+            version  = version
+        )
+
+        self.trace_yes = trace_yes
+        self.trace_no = trace_no
+
+    def onValueEscape( self ):
+        pass
+
 
 class VariableUsageProfile:
     def __init__( self, variable ):
@@ -110,9 +170,8 @@ class ConstraintCollectionBase:
         self.parent = parent
 
         # Variable assignments performed in here.
-        self.variable_targets = {}
         self.variable_versions = {}
-        self.variable_escaped = {}
+        self.variable_traces = {}
 
         self.variables = {}
 
@@ -277,46 +336,67 @@ class ConstraintCollectionBase:
         else:
             return statements_sequence
 
-    def _addVariableUsageInfo( self, variable, version, info ):
-        if version == 0:
-            # If we have no knowledge yet, use "0" still. We will treat it special
-            # depending on context. Unknown will "-1".
-            version = self.variable_versions.get( variable, 0 )
-        else:
-            self.variable_versions[ variable ] = version
+
+    def onVariableSet( self, target_node, value_friend ):
+        # Add a new trace, allocating a new version for the variable, and remember the value
+        # friend.
+        variable = target_node.getVariable()
+
+        while variable.isReference():
+            variable = variable.getReferenced()
+
+        assert not variable.isModuleVariableReference()
+
+        version  = target_node.getVariableVersion()
 
         key = variable, version
 
-        if key in self.variable_targets:
-            self.variable_targets[ key ].append( info )
-        else:
-            self.variable_targets[ key ] = [ info ]
-
-
-    def onVariableSet( self, target_node, value_friend ):
-        # Remember the node that updates plus the then current value in the trace.
-        self._addVariableUsageInfo(
-            variable = target_node.getVariable(),
-            version  = target_node.getVariableVersion(),
-            info     = ( target_node, value_friend )
+        self.variable_traces[ key ] = VariableAssignTrace(
+            target_node  = target_node,
+            variable     = variable,
+            version      = version,
+            value_friend = value_friend
         )
 
+        self.variable_versions[ variable ] = version
+
+        return key
+
     def onVariableUsage( self, ref_node ):
-        self._addVariableUsageInfo(
-            variable = ref_node.getVariable(),
-            version  = 0,
-            info     = ( ref_node )
-        )
+        variable = ref_node.getVariable()
+
+        while variable.isReference():
+            variable = variable.getReferenced()
+
+        assert not variable.isModuleVariableReference()
+
+        version  = self.variable_versions.get( variable, 0 )
+
+        key = variable, version
+
+        if key in self.variable_traces:
+            self.variable_traces[ key ].addUsage( ref_node )
+
+            return None
+        else:
+            self.variable_traces[ key ] = VariableReferenceTrace(
+                ref_node = ref_node,
+                variable = variable,
+                version  = version
+            )
+
+            self.variable_versions[ variable ] = version
+
+            return key
 
     def onVariableContentEscapes( self, variable ):
         version = self.variable_versions.get( variable, 0 )
 
-        if version != 0:
-            key = variable, version
+        key = variable, version
 
-            if key not in self.variable_escaped:
-                # Indicate when the variable value escaped.
-                self.variable_escaped[ key ] = len( self.variable_targets[ key ] )
+        if key in self.variable_traces:
+            # Indicate when the variable value escaped.
+            self.variable_traces[ key ].onValueEscape()
 
     def onExpression( self, expression, allow_none = False ):
         if expression is None and allow_none:
@@ -426,7 +506,7 @@ Removed assignment that has source that will raise."""
         if not variable.isModuleVariableReference() and \
              source.isExpressionVariableRef() and \
              source.getVariable() == variable:
-            if source.mayHaveSideEffects( self ):
+            if source.mayHaveSideEffects():
                 result = makeStatementExpressionOnlyReplacementNode(
                     expression = source,
                     node       = statement
@@ -485,18 +565,96 @@ Side effects of assignments promoted to statements."""
         return result
 
     def onStatement( self, statement ):
-        assert statement.isStatement(), statement
+        try:
+            assert statement.isStatement(), statement
 
-        new_statement, change_tags, change_desc = statement.computeStatement( self )
+            new_statement, change_tags, change_desc = statement.computeStatement( self )
 
-        if new_statement is not statement:
-            self.signalChange(
-                change_tags,
-                statement.getSourceReference(),
-                change_desc
+            if new_statement is not statement:
+                self.signalChange(
+                    change_tags,
+                    statement.getSourceReference(),
+                    change_desc
+                )
+
+            return new_statement
+        except Exception:
+            warning( "Problem with statement at %s:", statement.getSourceReference() )
+            raise
+
+    def mergeBranches( self, collection_yes, collection_no ):
+        def getParentVariableVersion( variable ):
+            return self.parent.getVariableVersions().get( variable, None )
+
+        def mergeSingleBranchChange( variable, version ):
+            key = variable, variable.allocateTargetNumber()
+
+            # Now we didn't find it in yes, so it's up to merging.
+            if variable in self.variable_versions:
+                trace_old = self.variable_traces[ variable, self.variable_versions[ variable ] ]
+            else:
+                trace_old = None
+
+            self.variable_traces[ key ] = VariableMergeTrace(
+                variable     = variable,
+                version      = key[1],
+                trace_yes    = self.variable_traces[ variable, version ],
+                trace_no     = trace_old
             )
 
-        return new_statement
+        if collection_yes is not None:
+            added_yes = collection_yes.getBranchOnlyTraces()
+
+            if collection_no is not None:
+                added_no = collection_no.getBranchOnlyTraces()
+
+                # Merge yes branch assignments with either existing ones from "no" branch,
+                # or with the original start point.
+                for yes_variable, yes_version in reversed( added_yes ):
+                    key = yes_variable, yes_variable.allocateTargetNumber()
+
+                    for no_variable, no_version in reversed( added_no ):
+                        if yes_variable == no_variable:
+                            self.variable_traces[ key ] = VariableMergeTrace(
+                                variable     = yes_variable,
+                                # TODO: Get this from ourselves.
+                                version      = key[1],
+                                trace_yes    = self.variable_traces[ yes_variable, yes_version ],
+                                trace_no     = self.variable_traces[ no_variable, no_version ]
+                            )
+
+                            break
+                    else:
+                        if yes_variable in self.variable_versions:
+                            trace_no = self.variable_traces[ yes_variable, self.variable_versions[ yes_variable ] ]
+                        else:
+                            trace_no = None
+
+                        self.variable_traces[ key ] = VariableMergeTrace(
+                            variable     = yes_variable,
+                            # TODO: Get this from ourselves.
+                            version      = key[1],
+                            trace_yes    = self.variable_traces[ yes_variable, yes_version ],
+                            trace_no     = trace_no
+                        )
+
+
+                # Merge no branch variables if their version is higher, meaning it's
+                # actually still a newer assignment, and wasn't merged yet.
+                for no_variable, no_version in reversed( added_no ):
+                    for yes_variable, yes_version in reversed( added_yes ):
+                        if yes_variable == no_variable:
+                            break
+                    else:
+                        mergeSingleBranchChange( no_variable, no_version )
+            else:
+                for yes_variable, yes_version in reversed( added_yes ):
+                    mergeSingleBranchChange( yes_variable, yes_version )
+        elif collection_no is not None:
+            added_no = collection_no.getBranchOnlyTraces()
+
+            for no_variable, no_version in reversed( added_no ):
+                mergeSingleBranchChange( no_variable, no_version )
 
 
 class ConstraintCollectionHandler( ConstraintCollectionBase ):
@@ -520,6 +678,14 @@ class ConstraintCollectionHandler( ConstraintCollectionBase ):
 
 
 class ConstraintCollectionBranch( ConstraintCollectionBase ):
+    def __init__( self, parent ):
+        ConstraintCollectionBase.__init__(
+            self,
+            parent = parent
+        )
+
+        self.branch_only_traces = OrderedSet()
+
     def process( self, branch ):
         assert branch.isStatementsSequence(), branch
 
@@ -528,6 +694,37 @@ class ConstraintCollectionBranch( ConstraintCollectionBase ):
         if result is not branch:
             branch.replaceWith( result )
 
+    def onVariableSet( self, target_node, value_friend ):
+        # Add a new trace, allocating a new version for the variable, and remember the value
+        # friend.
+        key = self.parent.onVariableSet(
+            target_node  = target_node,
+            value_friend = value_friend
+        )
+
+        # Remember the version, because it was added to this branch only, which matters
+        # for merge later.
+        self.branch_only_traces.add( key )
+
+        return key
+
+    def onVariableUsage( self, ref_node ):
+        key = self.parent.onVariableUsage(
+            ref_node = ref_node
+        )
+
+        if key is not None:
+            self.branch_only_traces.add( key )
+
+        return key
+
+    def getBranchOnlyTraces( self ):
+        return self.branch_only_traces
+
+    def mergeBranches( self, collection_yes, collection_no ):
+        # Branches in branches, should ask parent about merging them.
+        return self.parent.mergeBranches( collection_yes, collection_no )
+
 
 class ConstraintCollectionFunction( ConstraintCollectionBase, VariableUsageTrackingMixin ):
     def __init__( self, parent ):
@@ -623,7 +820,8 @@ class ConstraintCollectionModule( ConstraintCollectionBase, VariableUsageTrackin
                     source_ref     = module.getSourceReference(),
                     module_name    = module_package,
                     parent_package = None,
-                    level          = 1
+                    level          = 1,
+                    warn           = True
                 )
 
                 imported_module, added_flag = Recursion.recurseTo(
