@@ -54,13 +54,23 @@ class VariableClosureLookupVisitorPhase1( VisitorNoopMixin ):
     def onEnterNode( self, node ):
         if node.isExpressionTargetVariableRef():
             if node.getVariable() is None:
+                variable_name = node.getVariableName()
                 provider = node.getParentVariableProvider()
 
-                node.setVariable(
-                    provider.getVariableForAssignment(
-                        variable_name = node.getVariableName()
-                    )
+                variable = provider.getVariableForAssignment(
+                    variable_name = variable_name
                 )
+
+                # Inside an exec, we need to ignore global declarations that are
+                # not ours, so we replace it with ours, unless it came from an
+                # 'global' declaration inside the exec
+                if node.source_ref.isExecReference() and not provider.isPythonModule():
+                    if variable.isModuleVariableReference() and not variable.isFromExecStatement():
+                        variable = provider.providing[ variable_name ] = provider.createProvidedVariable(
+                            variable_name = variable_name
+                        )
+
+                node.setVariable( variable )
         elif node.isExpressionVariableRef():
             if node.getVariable() is None:
                 provider = node.getParentVariableProvider()
@@ -126,7 +136,44 @@ class VariableClosureLookupVisitorPhase1( VisitorNoopMixin ):
                         break
                     else:
                         seen_function = True
+        # Check if continue and break are properly in loops. If not, raise a syntax error.
+        elif node.isStatementBreakLoop() or node.isStatementContinueLoop():
+            current = node
 
+            while True:
+                if current.isPythonModule() or current.isExpressionFunctionBody():
+                    if node.isStatementContinueLoop():
+                        message = "'continue' not properly in loop"
+                        col_offset   = 16 if python_version >= 300 else None
+                        display_line = True
+                        source_line  = None
+                    else:
+                        message = "'break' outside loop"
+
+                        if isFullCompat():
+                            col_offset   = 2 if python_version >= 300 else None
+                            display_line = True
+                            source_line  = "" if python_version >= 300 else None
+                        else:
+                            col_offset   = 13
+                            display_line = True
+                            source_line  = None
+
+                    source_ref = node.getSourceReference()
+                    # source_ref.line += 1
+
+                    SyntaxErrors.raiseSyntaxError(
+                        message,
+                        source_ref   = node.getSourceReference(),
+                        col_offset   = col_offset,
+                        display_line = display_line,
+                        source_line  = source_line
+                    )
+
+                current = current.getParent()
+
+                if current.isStatementLoop():
+                    break
 
     def onLeaveNode( self, node ):
         # Return statements in generators are not really that, instead they are exception
@@ -185,15 +232,18 @@ class VariableClosureLookupVisitorPhase2( VisitorNoopMixin ):
 
                 parent_provider = provider.getParentVariableProvider()
 
-                while parent_provider.isExpressionFunctionBody() and parent_provider.isClassDictCreation():
+                while parent_provider.isExpressionFunctionBody() and \
+                      parent_provider.isClassDictCreation():
                     parent_provider = parent_provider.getParentVariableProvider()
 
-                if parent_provider.isExpressionFunctionBody() and parent_provider.isUnqualifiedExec():
+                if parent_provider.isExpressionFunctionBody() and \
+                   parent_provider.isUnqualifiedExec():
                     lines = open( node.source_ref.getFilename(), "rU" ).readlines()
                     exec_line_number = parent_provider.getExecSourceRef().getLineNumber()
 
                     raise SyntaxError(
-                        "unqualified exec is not allowed in function '%s' it contains a nested function with free variables" % parent_provider.getName(), # pylint: disable=C0301
+                        """unqualified exec is not allowed in function '%s' it contains \
+a nested function with free variables""" % parent_provider.getName(),
                         (
                             node.source_ref.getFilename(),
                             exec_line_number,
